uniform float windWaveCounter;
uniform float windWaveCounterHighFreq;
uniform float waterWaveCounter;
uniform float windSpeed;
uniform vec3 playerpos;
uniform float globalWarpIntensity;

uniform float glitchStrength = 0;
uniform float windWaveIntensity = 1;

// Value noise by https://www.shadertoy.com/view/4sfGzS
vec3 ghash( vec3 p ) // replace this by something better
{
	p = vec3( dot(p,vec3(127.1,311.7, 74.7)),
			  dot(p,vec3(269.5,183.3,246.1)),
			  dot(p,vec3(113.5,271.9,124.6)));

	return -1.0 + 2.0*fract(sin(p)*43758.5453123);
}

float gnoise( in vec3 p )
{
    vec3 i = floor( p );
    vec3 f = fract( p );
	
	vec3 u = f*f*(3.0-2.0*f);

	// Added 1.2 here because our old noise was stronger
    return 1.2 * mix( mix( mix( dot( ghash( i + vec3(0.0,0.0,0.0) ), f - vec3(0.0,0.0,0.0) ), 
                          dot( ghash( i + vec3(1.0,0.0,0.0) ), f - vec3(1.0,0.0,0.0) ), u.x),
                     mix( dot( ghash( i + vec3(0.0,1.0,0.0) ), f - vec3(0.0,1.0,0.0) ), 
                          dot( ghash( i + vec3(1.0,1.0,0.0) ), f - vec3(1.0,1.0,0.0) ), u.x), u.y),
                mix( mix( dot( ghash( i + vec3(0.0,0.0,1.0) ), f - vec3(0.0,0.0,1.0) ), 
                          dot( ghash( i + vec3(1.0,0.0,1.0) ), f - vec3(1.0,0.0,1.0) ), u.x),
                     mix( dot( ghash( i + vec3(0.0,1.0,1.0) ), f - vec3(0.0,1.0,1.0) ), 
                          dot( ghash( i + vec3(1.0,1.0,1.0) ), f - vec3(1.0,1.0,1.0) ), u.x), u.y), u.z );
}

vec4 applyLiquidWarping(bool windAffected, vec4 worldPos, float div) {
	#if WAVINGSTUFF == 1
	vec3 noisepos = vec3((worldPos.x + playerpos.x) / 3, (worldPos.z + playerpos.z) / 3, waterWaveCounter / 8 + (windAffected ? windWaveCounter / 6 : 0));
	worldPos.y += gnoise(noisepos) / div;
	#endif
	
	return worldPos;
}

vec4 applyVertexWarping(int renderFlags, vec4 worldPos) {

	#if WAVINGSTUFF == 1

	// bit 12 is the water waving on/off flag
	if ((renderFlags & 0x1000) > 0) {
		vec3 noisepos = vec3((worldPos.x + playerpos.x) / 3, (worldPos.z + playerpos.z) / 3, waterWaveCounter / 8 + windWaveCounter / 6);
		worldPos.y += gnoise(noisepos) / 10;
	} else {
		// Coopers reed water layer has bit 11 and 12 set. Lets just ignore bit 11 in this case
		
		// bit 11 is the grass swaying on/off flag
		if ((renderFlags & 0x800) > 0 || (renderFlags & 0x8000000) > 0) {
		
			bool isLeaves = ((renderFlags & 0x8000000) > 0);
			
			float div = 30;            
			
			float strength = (0.5 + (isLeaves ? 0 : 0.5)) * windWaveIntensity * (1 + windSpeed);
			
			float x = windWaveCounterHighFreq + worldPos.x + playerpos.x;
			float y = windWaveCounterHighFreq + worldPos.y + playerpos.y;
			float z = windWaveCounterHighFreq + worldPos.z  + playerpos.z;
			
			float bendNoise = windSpeed * 0.2 + 0.8 * gnoise(vec3((worldPos.x + playerpos.x) / 10, (worldPos.z + playerpos.z) / 10, windWaveCounter/4));
			
			uint flagsbla = uint(renderFlags);
			int groundOffset = int((flagsbla >> 28)) & 0x7;
			
			float heightBend = (isLeaves ? max(groundOffset, 0) / 10.0 : 0.7);
			
			float bend = windSpeed * (1 + bendNoise) * heightBend * windWaveIntensity;
			bend = min(4, bend);
			
			if ((renderFlags & 0x4000) > 0) {
				bend /= 3;
				div *= 1.5;
			}

			// Curve visualized: http://fooplot.com/#W3sidHlwZSI6MCwiZXEiOiIyKnNpbih4LzgpK3Npbih4LzIpK3NpbigwLjUrMip4KStzaW4oMSszKngpIiwiY29sb3IiOiIjMDAwMDAwIn0seyJ0eXBlIjoxMDAwLCJ3aW5kb3ciOlsiLTI0Ljc5NTUzMjIyNjU2MjQ4NiIsIjI0Ljc5NTUzMjIyNjU2MjQ4NiIsIi0xNS4yNTg3ODkwNjI0OTk5OTEiLCIxNS4yNTg3ODkwNjI0OTk5OTEiXX1d
			
			worldPos.x += bend + strength * (sin(x*10)/120 + (2 * sin(x/2) + sin(x + y) + sin(0.5 + 4*x + 2*y) + sin(1 + 6*x + 3*y)/3) / div);
			worldPos.y += strength * (sin(5*y)/15 + cos(10*x) / 10 + sin(3*z)/2 + cos(x*2)/2.2)/div;
			worldPos.z += strength * ((2 * sin(z/4) + sin(z + 3 * y) + sin(0.5 + 4*z + 2*y) + sin(1 + 6*z + y)/3) / div);
		}
	}
	
	float str = max(0, glitchStrength - 0.1);
	if (str > 0) {
		str *= min(100, length(worldPos) * glitchStrength);
		worldPos.x += str * gnoise(vec3((worldPos.x+playerpos.x)/10, (worldPos.z+playerpos.z)/10, windWaveCounter/6)) / 5;
		worldPos.y += str * gnoise(vec3((worldPos.x+playerpos.x)/10, (worldPos.z+playerpos.z)/10, windWaveCounter/10)) / 5;
		worldPos.z += str * gnoise(vec3((worldPos.x+playerpos.x)/10, (worldPos.z+playerpos.z)/10, windWaveCounter/3.5)) / 5;
	}
	
	if (globalWarpIntensity > 0) {
		float x = max(0, (mod(20*windWaveCounter, 30)) + (worldPos.x + playerpos.x) / 5.0 + (worldPos.y + playerpos.y) / 8.0 - 40);
		worldPos.x += (sin(x / 2) + sin(0.5 + 2*x) + sin(1 + 3*x)/3) / 30.0 * globalWarpIntensity;
		
		worldPos.z += (cos(x / 3) + cos(0.2 + 2.2*x) + cos(1 + 4*x)/3) / 30.0 * globalWarpIntensity;
	}
	
	#endif
		
	return worldPos;
}




